/***********************************************************************
 *
 * linux/arch/arm/mach-ns9750/arch.c
 *
 * Copyright (C) 2006,2008 Atmark Techno, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 ***********************************************************************/
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/module.h>
#include <linux/device.h>

#include <asm/io.h>

#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/flash.h>

#include <asm/arch/ns9750_config.h>
#include <asm/arch/ns9750_sys.h>
#include <asm/arch/ns9750_mem.h>
#include <asm/arch/ns9750_eth.h>
#include <asm/arch/ns9750_bbus_bridge.h>
#include <asm/arch/ns9750_bbus.h>
#include <asm/arch/ns9750_usb.h>
#include <asm/arch/ns9750_i2c.h>
#include <asm/arch/ns9750_lcd.h>
#include <asm/arch/ns9750_ser.h>
#include <asm/arch/ns9750_bsd.h>
#include <asm/arch/ns9750_pci.h>
#include <asm/arch/irqs.h>
#include <asm/arch/time.h>

#if defined(CONFIG_APP_PLD_MISC)
#include <asm/arch/armadillo3x0_misc.h>
#endif

#if (VMALLOC_END >= 0xf8000000)
/* Documentation/arm/memory.txt leaves platform space from
   from VMALLOC_END to 0xff000000. */
# error VMALLOC_END clashes with peripheral pages
#endif

static struct map_desc standard_io_desc[] __initdata = {
	{ NS_SYS_MODULE_BASE, NS_SYS_MODULE_BASE_PA,
	  NS_SYS_MODULE_SIZE, MT_DEVICE },
	{ NS_MEM_MODULE_BASE, NS_MEM_MODULE_BASE_PA,
	  NS_MEM_MODULE_SIZE, MT_DEVICE },
	{ NS_ETH_MODULE_BASE, NS_ETH_MODULE_BASE_PA,
	  NS_ETH_MODULE_SIZE, MT_DEVICE },
	{ NS_BBUS_BRIDGE_MODULE_BASE, NS_BBUS_BRIDGE_MODULE_BASE_PA,
	  NS_BBUS_BRIDGE_MODULE_SIZE, MT_DEVICE },
	{ NS_BBUS_MODULE_BASE, NS_BBUS_MODULE_BASE_PA,
	  NS_BBUS_MODULE_SIZE + NS_USB_MODULE_SIZE, MT_DEVICE },
	{ NS_BBUS_UTILITY_BASE, NS_BBUS_UTILITY_BASE_PA,
	  NS_BBUS_UTILITY_SIZE, MT_DEVICE },
	{ NS_I2C_MODULE_BASE, NS_I2C_MODULE_BASE_PA,
	  NS_I2C_MODULE_SIZE, MT_DEVICE },
	{ NS_LCD_MODULE_BASE, NS_LCD_MODULE_BASE_PA,
	  NS_LCD_MODULE_SIZE, MT_DEVICE },
	{ NS_SER_MODULE_BASE, NS_SER_MODULE_BASE_PA,
	  NS_SER_MODULE_SIZE, MT_DEVICE },
	{ NS_BSD_MODULE_BASE, NS_BSD_MODULE_BASE_PA,
	  NS_BSD_MODULE_SIZE, MT_DEVICE },
	{ NS_PCI_IO_BASE, NS_PCI_IO_BASE_PA, NS_PCI_IO_SIZE, MT_DEVICE },
	{ NS_PCI_CONFIG_ADDR_BASE, NS_PCI_CONFIG_ADDR_BASE_PA,
	  NS_PCI_CONFIG_ADDR_SIZE, MT_DEVICE },
	{ NS_PCI_CONFIG_DATA_BASE, NS_PCI_CONFIG_DATA_BASE_PA,
	  NS_PCI_CONFIG_DATA_SIZE, MT_DEVICE },
	{ NS_PCI_MODULE_BASE, NS_PCI_MODULE_BASE_PA,
	  NS_PCI_MODULE_SIZE, MT_DEVICE },
#if defined(CONFIG_APP_PLD_MISC)
	{ A3X0_MISC_MODULE_BASE, A3X0_MISC_MODULE_BASE_PA,
	  A3X0_MISC_MODULE_SIZE, MT_DEVICE },
#endif
#if defined(CONFIG_APP_PLD_IDE)
	{ A3X0_IDE_MODULE_BASE, A3X0_IDE_MODULE_BASE_PA,
	  A3X0_IDE_MODULE_SIZE, MT_DEVICE },
#endif
};

static struct resource resources[] = {
	{
		.name	= "dma_0",
		.start	= (NS_BBUS_DMA_DESCR( 0, 0 )
			   - NS_BBUS_MODULE_BASE + NS_BBUS_MODULE_BASE_PA),
		.end	= (NS_BBUS_DMA_STAT( 0, 15 ) + 3
			   - NS_BBUS_MODULE_BASE + NS_BBUS_MODULE_BASE_PA)
	}, {
		.name	= "dma_1",
		.start	= (NS_BBUS_DMA_DESCR( 1, 0 )
			   - NS_BBUS_MODULE_BASE + NS_BBUS_MODULE_BASE_PA),
		.end	= (NS_BBUS_DMA_STAT( 1, 15 ) + 3
			   - NS_BBUS_MODULE_BASE + NS_BBUS_MODULE_BASE_PA)
	}, {
		.name = NULL
	}
};

static struct resource ns9750_usbhost_resources [] = {
        [0] = {
                .start  = (NS_USB_HCR
			   - NS_USB_MODULE_BASE + NS_USB_MODULE_BASE_PA),
                .end    = (NS_USB_HCR_PORT_SAT
			   - NS_USB_MODULE_BASE + NS_USB_MODULE_BASE_PA),
                .flags  = IORESOURCE_MEM,
        },
	[1] = {
		.start = (IRQ_USB_HOST),
		.end = (IRQ_USB_HOST),
		.flags = IORESOURCE_IRQ,
	},
};

static u64 ns9750_usbhost_dma_mask = 0xffffffffUL;

static struct platform_device ns9750_usbhost_device = {
        .name           = "ns9750-ohci",
        .id             = 0,
        .dev            = {
                .dma_mask = &ns9750_usbhost_dma_mask,
                .coherent_dma_mask = 0xffffffffUL,
        },
        .num_resources  = ARRAY_SIZE (ns9750_usbhost_resources),
        .resource       = ns9750_usbhost_resources,
};

static struct resource ns9750_ethernet_resources[] = {
	[0] = {
	  .start	= NS_ETH_MODULE_BASE_PA,
	  .end		= NS_ETH_MODULE_BASE_PA + 0xffff,
	  .flags	= IORESOURCE_MEM,
	},
	[1] = {
	  .start	= IRQ_ETH_RX,
	  .end		= IRQ_ETH_PHY,
	  .flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device ns9750_ethernet_device = {
	.name		= "ns9750ethernet",
	.id		= 0,
	.num_resources	= 2,
	.resource	= ns9750_ethernet_resources,
};

static struct resource ns9750_raster_resources[] = {
  [0] = {
    .start	= NS_LCD_MODULE_BASE_PA,
    .end	= NS_LCD_MODULE_BASE_PA + 0x0fffff,
    .flags	= IORESOURCE_MEM,
  },
};

static struct platform_device ns9750_raster_device = {
  .name	= "ns9750fb",
  .id	= 0,
  .resource	= ns9750_raster_resources,
  .num_resources	= 1,
};

static struct platform_device *ns9750_devices[] __initdata = {
	&ns9750_ethernet_device,
	&ns9750_usbhost_device,
	&ns9750_raster_device,
};

#if defined(CONFIG_MACH_ARMADILLO300)
static struct flash_platform_data armadillo3x0_flash_data = {
        .map_name       = "cfi_probe",
        .width          = 2,
};

static struct resource armadillo3x0_flash_resource = {
        .start          = 0x50000000,
        .end            = 0x507fffff,
        .flags          = IORESOURCE_MEM,
};

static struct platform_device armadillo3x0_flash_device = {
	.name		= "armadillo3x0-nor",
	.id		= 0,
	.dev		= {
		.platform_data	= &armadillo3x0_flash_data,
	},
	.num_resources	= 1,
	.resource	= &armadillo3x0_flash_resource,
};

static struct platform_device *armadillo3x0_devices[] __initdata = {
	&armadillo3x0_flash_device,
};
#endif

static unsigned int _ns_sys_clock_freq;

spinlock_t ns_gpio_lock;	/* use it to lock all GPIO accesses */

extern void __init ns_init_irq( void );
static void __init ns_map_io( void );
static int __init ns_init( void );

static void __init ns_map_io(void)
{
	_ns_sys_clock_freq = NS_SYS_CLK_FREQ;

	iotable_init( standard_io_desc, ARRAY_SIZE( standard_io_desc ) );

	printk( "NS9750 Rev. %i, running at %u MHz\n",
		(readl( NS_SYS_MISC ) >> 24) & 0xff,
		_ns_sys_clock_freq/2000000 );
}

static void set_arbiter(int channel, u8 data)
{
	unsigned long reg = (channel & ~0x03);
	unsigned char locate = (3 - (channel & 0x03));
	unsigned long brc;
	brc = readl(NS_SYS_BRC_BASE + reg);
	brc &= ~(0xff << (locate << 3));
	brc |= (data << (locate << 3));
	writel(brc, NS_SYS_BRC_BASE + reg);
}

#define ARBITER_DEFAULT 0
int ns_set_arbiter(int mode)
{
	u8 data[16];
	int channel;

	memset(data, 0, 16);

	switch (mode) {
	case ARBITER_DEFAULT:
	  /*
	   * ARM : 50%,  Eth Rx : 8.3%, Eth Tx : 8.3%,
	   * PCI : 8.3%, BBus   : 8.3%, LCD    : 16.6%,
	   */
	  data[0] = data[2] = data[4] = data[6] =
	    (NS_SYS_BRC_CEB | NS_SYS_BRC_BRF_50 | NS_SYS_BRC_HMSTR_ARM);
	  data[8] =
	    (NS_SYS_BRC_CEB | NS_SYS_BRC_BRF_100 | NS_SYS_BRC_HMSTR_ARM);
	  data[1] = 
	    (NS_SYS_BRC_CEB | NS_SYS_BRC_BRF_50 | NS_SYS_BRC_HMSTR_ETHRX);
	  data[3] = 
	    (NS_SYS_BRC_CEB | NS_SYS_BRC_BRF_50 | NS_SYS_BRC_HMSTR_ETHTX);
	  data[5] = 
	    (NS_SYS_BRC_CEB | NS_SYS_BRC_BRF_50 | NS_SYS_BRC_HMSTR_PCI);
	  data[7] = 
	    (NS_SYS_BRC_CEB | NS_SYS_BRC_BRF_50 | NS_SYS_BRC_HMSTR_BBUS);
	  data[9] = 
	    (NS_SYS_BRC_CEB | NS_SYS_BRC_BRF_100 | NS_SYS_BRC_HMSTR_LCD);
	  for (channel=0; channel<16; channel++) {
	  	set_arbiter(channel, data[channel]);
	  }
	  break;
	default:
	  return -1;
	}
	return 0;
}
EXPORT_SYMBOL(ns_set_arbiter);

static int __init ns_init( void )
{
	int i = 0;

	ns_set_arbiter(ARBITER_DEFAULT);

	platform_add_devices(ns9750_devices, ARRAY_SIZE(ns9750_devices));
#if defined(CONFIG_MACH_ARMADILLO300)
	platform_add_devices(armadillo3x0_devices, ARRAY_SIZE(armadillo3x0_devices));
#endif

	spin_lock_init( &ns_gpio_lock );

	while( resources[ i ].name != NULL ) {
		request_resource( &iomem_resource, &resources[ i++ ] );
	}

	return 0;
}

/***********************************************************************
 * @Function:
 * @Return:
 * @Descr:  provides system clock for drivers
 ***********************************************************************/

unsigned int ns_sys_clock_freq( void )
{
	return _ns_sys_clock_freq;
}

EXPORT_SYMBOL( ns_sys_clock_freq );

arch_initcall( ns_init );

#if defined(CONFIG_MACH_ARMADILLO300)
MACHINE_START(ARMADILLO300, "ARMADILLO300")
     MAINTAINER("Atmark Techno, Inc.")
     BOOT_MEM(PHYS_OFFSET, NS_BBUS_MODULE_BASE_PA, NS_BBUS_MODULE_BASE)
     BOOT_PARAMS(PHYS_OFFSET + 0x100)
     MAPIO(ns_map_io)
     INITIRQ(ns_init_irq)
     .timer = &ns_timer,
MACHINE_END
#endif
