
#include <linux/config.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>
#include <linux/fb.h>
#include <linux/ioport.h>
#include <linux/dma-mapping.h>

#include <asm/arch/ns9750_lcd.h>
#include <asm/arch/ns9750_bbus.h>

#define DRIVER_NAME "ns9750fb"
#define REVISION "Rev.2 (2007/02/20)"
#define AUTHOR "Atmark Techno, Inc."
#define DESCRIPTION "NS9750 frame buffer driver"

#undef DEBUG
//#define DEBUG
#if defined(DEBUG)
#define DEBUG_FUNC()        printk(DRIVER_NAME ": %s()\n", __FUNCTION__)
#define DEBUG_INFO(args...) printk(DRIVER_NAME ": " args)
#else
#define DEBUG_FUNC()
#define DEBUG_INFO(args...)
#endif
#define PRINT_INFO(args...)  printk(KERN_INFO DRIVER_NAME ": " args)
#define PRINT_DEBUG(args...) printk(KERN_DEBUG DRIVER_NAME ": " args)
#define PRINT_ERR(args...)   printk(KERN_ERR DRIVER_NAME ": " args)

struct ns9750fb_gpio
{
	int gpio;
	int func;
	int flag; /* 0: [1..16]bpp, 1: [1..16]&24bpp */
};

static struct ns9750fb_gpio gpios[] = {
	{ 18, NS_GPIO_CFG_FUNC_1, 0 }, /*LCD PWR*/
	{ 19, NS_GPIO_CFG_FUNC_1, 0 }, /*LCD HSYNC*/
	{ 20, NS_GPIO_CFG_FUNC_1, 0 }, /*LCD CLOCK*/
	{ 21, NS_GPIO_CFG_FUNC_1, 0 }, /*LCD FP*/
	{ 22, NS_GPIO_CFG_FUNC_1, 0 }, /*LCD AC*/
	{ 23, NS_GPIO_CFG_FUNC_1, 0 }, /*LCD LE*/
	{ 24, NS_GPIO_CFG_FUNC_1, 0 }, /*LCD D0*/
	{ 25, NS_GPIO_CFG_FUNC_1, 0 }, /*LCD D1*/
	{ 26, NS_GPIO_CFG_FUNC_1, 0 }, /*LCD D2*/
	{ 27, NS_GPIO_CFG_FUNC_1, 0 }, /*LCD D3*/
	{ 28, NS_GPIO_CFG_FUNC_1, 0 }, /*LCD D4*/
	{ 29, NS_GPIO_CFG_FUNC_1, 0 }, /*LCD D5*/
	{ 30, NS_GPIO_CFG_FUNC_1, 0 }, /*LCD D6*/
	{ 31, NS_GPIO_CFG_FUNC_1, 0 }, /*LCD D7*/
	{ 32, NS_GPIO_CFG_FUNC_2, 0 }, /*LCD D8*/
	{ 33, NS_GPIO_CFG_FUNC_2, 0 }, /*LCD D9*/
	{ 34, NS_GPIO_CFG_FUNC_2, 0 }, /*LCD D10*/
	{ 35, NS_GPIO_CFG_FUNC_2, 0 }, /*LCD D11*/
	{ 36, NS_GPIO_CFG_FUNC_2, 0 }, /*LCD D12*/
	{ 37, NS_GPIO_CFG_FUNC_2, 0 }, /*LCD D13*/
	{ 38, NS_GPIO_CFG_FUNC_2, 0 }, /*LCD D14*/
	{ 39, NS_GPIO_CFG_FUNC_2, 0 }, /*LCD D15*/
	{ 40, NS_GPIO_CFG_FUNC_2, 0 }, /*LCD D16*/
	{ 41, NS_GPIO_CFG_FUNC_2, 0 }, /*LCD D17*/
	{ 42, NS_GPIO_CFG_FUNC_2, 1 }, /*LCD D18*/
	{ 43, NS_GPIO_CFG_FUNC_2, 1 }, /*LCD D19*/
	{ 44, NS_GPIO_CFG_FUNC_2, 1 }, /*LCD D20*/
	{ 45, NS_GPIO_CFG_FUNC_2, 1 }, /*LCD D21*/
	{ 46, NS_GPIO_CFG_FUNC_2, 1 }, /*LCD D22*/
	{ 47, NS_GPIO_CFG_FUNC_2, 1 }, /*LCD D23*/
};

struct ns9750fb_videotiming
{
	const char *name[7];
	void (*configure)(struct fb_info *info);
	void (*on)(struct fb_info *info);
        void (*off)(struct fb_info *info);

        unsigned long hres;
        unsigned long hfp;
        unsigned long hsync;
        unsigned long hbp;

        unsigned long vres;
        unsigned long vfp;
        unsigned long vsync;
        unsigned long vbp;

	unsigned long pcd;
};

#define TIMING(_hres, _hfp, _hsync, _hbp, _vres, _vfp, _vsync, _vbp) \
	.hres	= _hres,  \
	.hfp	= _hfp,   \
	.hsync	= _hsync, \
	.hbp	= _hbp,   \
	.vres	= _vres,  \
	.vfp	= _vfp,   \
	.vsync	= _vsync, \
	.vbp	= _vbp

static struct ns9750fb_videotiming modedb[] = {
	{
	  .name[0]	= "CRT-VGA",
	  .name[1]	= "CRT-640x480",
	  .name[2]	= "CRT-640x480@60",
	  .name[3]	= NULL,
	  
	  .pcd	= 2,
	  TIMING(640, 30, 75, 45, 480, 10, 2, 33),
	},
	{
	  .name[0]	= "CRT-SVGA",
	  .name[1]	= "CRT-800x600",
	  .name[2]	= "CRT-800x600@72",
	  .name[3]	= NULL,
	  
	  .pcd	= 0,
	  TIMING(800, 55, 118, 63, 600, 37, 6, 23),
	},
#if 0
	{
	  .name[0]	= "CRT-XGA",
	  .name[1]	= "CRT-1024x768",
	  .name[2]	= "CRT-1024x768@85",
	  .name[3]	= NULL,
	  
	  .pcd	= -1, /* Enable Bypass pixel clock divider */
	  TIMING(1024, 88, 96, 248, 768, 1, 3, 36),
	},
#endif
};

dma_addr_t fb_phys;

/* Default Mode */
static char *modestr = NULL;
static int mode = -1;
static int bpp = -1;
static int enable = CONFIG_FB_NS9750_DEFAULT_STATE;

static inline void
ns9750fb_palette_write(unsigned regno, unsigned red, unsigned green,
		       unsigned blue, unsigned trans)
{
	unsigned int palette, val;

	DEBUG_FUNC();

	if(regno > 256)
		return;

	palette =  ((blue & 0xf800) >> 11);
	palette |= ((green & 0xf800) >> 6);
	palette |= ((red & 0xf800) >> 1);

	val = readl(NS_LCD_PALETTE + (4 * (regno >> 1)));
	val &= ((regno & 1) ? 0x0000ffff : 0xffff0000);

	val |= (palette << ((regno & 1) ? 16 : 0));
	writel(val, NS_LCD_PALETTE + (4 * (regno >> 1)));
}

static int
ns9750fb_setcolreg(unsigned regno, unsigned red, unsigned green, unsigned blue,
		   unsigned transp, struct fb_info *info)
{
	DEBUG_FUNC();

#define CNVT_TOHW(val,width) ((((val)<<(width))+0x7FFF-(val))>>16)

        switch (info->fix.visual) {
        case FB_VISUAL_PSEUDOCOLOR:
		ns9750fb_palette_write(regno, red, green, blue, transp);
                break;
        case FB_VISUAL_TRUECOLOR:
                if (regno >= 16)
                        return 1;

                red = CNVT_TOHW(red, info->var.red.length);
                green = CNVT_TOHW(green, info->var.green.length);
                blue = CNVT_TOHW(blue, info->var.blue.length);
                transp = CNVT_TOHW(transp, info->var.transp.length);
                ((u32 *) (info->pseudo_palette))[regno] =
                  (red << info->var.red.offset) |
                  (green << info->var.green.offset) |
                  (blue << info->var.blue.offset) |
                  (transp << info->var.transp.offset);
                break;
        case FB_VISUAL_DIRECTCOLOR:
                red = CNVT_TOHW(red, 8);
                green = CNVT_TOHW(green, 8);
                blue = CNVT_TOHW(blue, 8);
                transp = CNVT_TOHW(transp, 8);
                break;
        }
#undef CNVT_TOHW

  return 0;
}

static int
ns9750fb_set_par(struct fb_info *info)
{
	DEBUG_FUNC();
        switch (info->var.bits_per_pixel) {
        case 8:
                info->fix.visual = FB_VISUAL_PSEUDOCOLOR;
                break;

        case 16:
        case 24:
        case 32:
                info->fix.visual = FB_VISUAL_TRUECOLOR;
                break;

        default:
                return -EINVAL;
        }

        info->fix.line_length = (info->var.xres * info->var.bits_per_pixel) / 8;

        return 0;

}

static int ns9750fb_enable(struct fb_info *info);
static int ns9750fb_disable(struct fb_info *info);
static int
ns9750fb_blank(int blank_mode, struct fb_info *info)
{
	DEBUG_FUNC();

	if (blank_mode) {
		ns9750fb_disable(info);
	} else {
		if(enable)
			ns9750fb_enable(info);
	}
	return 0;
}

static int
ns9750fb_ioctl(struct inode *inode, struct file *file,
	       unsigned int cmd, unsigned long arg, struct fb_info *info)
{
	DEBUG_FUNC();
	DEBUG_INFO("cmd: %d\n", cmd);
	switch(cmd){
	default:
		return -EFAULT;
	}
}

static struct fb_ops ns9750fb_ops = {
	.owner		= THIS_MODULE,
	.fb_setcolreg	= ns9750fb_setcolreg,
	.fb_set_par	= ns9750fb_set_par,
	.fb_blank	= ns9750fb_blank,
	.fb_fillrect	= cfb_fillrect,
	.fb_copyarea	= cfb_copyarea,
	.fb_imageblit	= cfb_imageblit,
	.fb_cursor	= soft_cursor,
	.fb_ioctl	= ns9750fb_ioctl,
};

static int 
ns9750fb_gpio_request(int flag)
{
	int i;
	DEBUG_FUNC();
	for(i=0; i<ARRAY_SIZE(gpios); i++){
		if(gpios[i].flag <= flag){
			if(!request_gpio(gpios[i].gpio, 1, DRIVER_NAME))
				goto error;
		}
	}

	for(i=0; i<ARRAY_SIZE(gpios); i++){
		if(gpios[i].flag <= flag){
			set_gpio_cfg(gpios[i].gpio, gpios[i].func);
		}
	}

	return 0;

 error:
	while(i--)
		release_gpio(gpios[i].gpio, 1);
	
	return -EBUSY;
}

static int
ns9750fb_gpio_release(int flag)
{
	int i;
	DEBUG_FUNC();

	for(i=0; i<ARRAY_SIZE(gpios); i++){
		if(gpios[i].flag <= flag){
			release_gpio(gpios[i].gpio, 1);
		}
	}
	return 0;
}

static int
ns9750fb_enable(struct fb_info *info)
{
	unsigned long timing[4] = {0, 0, 0, 0};
	unsigned long control = 0;
	spinlock_t lock;
	
	DEBUG_FUNC();
	
	spin_lock_init(lock);
	
	spin_lock_irq(&lock);
	
	timing[0] |= NS_LCD_TIMING_0_HBP(info->var.right_margin - 1);
	timing[0] |= NS_LCD_TIMING_0_HFP(info->var.left_margin  - 1);
	timing[0] |= NS_LCD_TIMING_0_HSW(info->var.hsync_len    - 1);
	timing[0] |= NS_LCD_TIMING_0_PPL((info->var.xres >> 4)  - 1);
	
	timing[1] |= NS_LCD_TIMING_1_VBP(info->var.lower_margin    );
	timing[1] |= NS_LCD_TIMING_1_VFP(info->var.upper_margin    );
	timing[1] |= NS_LCD_TIMING_1_VSW(info->var.vsync_len    - 1);
	timing[1] |= NS_LCD_TIMING_1_LPP(info->var.yres         - 1);
	
	timing[2] |= NS_LCD_TIMING_2_CPL(info->var.xres         - 1);
	timing[2] |= NS_LCD_TIMING_2_IHS;
	timing[2] |= NS_LCD_TIMING_2_IVS;
	
	if(modedb[mode].pcd != -1){
	  timing[2] |= NS_LCD_TIMING_2_PCD(modedb[mode].pcd          );
	}else{
	  timing[2] |= NS_LCD_TIMING_2_BCD;
	}
	
	timing[3] |= NS_LCD_TIMING_3_LED(0                         );
	
	switch(bpp){
	case 1:  control |= NS_LCD_CONTROL_BPP_1; break;
	case 2:  control |= NS_LCD_CONTROL_BPP_2; break;
	case 4:  control |= NS_LCD_CONTROL_BPP_4; break;
	case 8:  control |= NS_LCD_CONTROL_BPP_8; break;
	case 16: control |= NS_LCD_CONTROL_BPP_16; break;
	case 24:
	case 32: control |= NS_LCD_CONTROL_BPP_24; break;
	default: break;
	}
	
	control   |= (NS_LCD_CONTROL_WATERMARK |
		      NS_LCD_CONTROL_PWR |
		      NS_LCD_CONTROL_BGR |
		      NS_LCD_CONTROL_TFT |
		      NS_LCD_CONTROL_EN);
	
	writel(0        , NS_LCD_CONTROL);
	
	writel(info->fix.smem_start, NS_LCD_UPBASE);
	
	writel(timing[0], NS_LCD_TIMING_0);
	writel(timing[1], NS_LCD_TIMING_1);
	writel(timing[2], NS_LCD_TIMING_2);
	writel(timing[3], NS_LCD_TIMING_3);
	writel(control  , NS_LCD_CONTROL);
	
	spin_unlock_irq(&lock);
	
	return 0;
}

static int
ns9750fb_disable(struct fb_info *info)
{
	spinlock_t lock;

	DEBUG_FUNC();

	spin_lock_init(lock);

	spin_lock_irq(&lock);
	
	writel(0, NS_LCD_CONTROL);
	
	spin_unlock_irq(&lock);
	
	return 0;
}

static int
ns9750fb_configure(struct fb_info *info)
{
	unsigned long htotal, hres, hfp, hsync, hbp;
	unsigned long vtotal, vres, vfp, vsync, vbp;
	unsigned long fb_size, clk;
	void *fb_log;
	
	DEBUG_FUNC();

	fb_size = modedb[mode].hres * modedb[mode].vres * (bpp / 8);
	fb_log = dma_alloc_writecombine(NULL, fb_size, &fb_phys, GFP_KERNEL);
	if (!fb_log)
		return -ENOMEM;

	if(modedb[mode].pcd != -1){
		clk = (AHB_CLK_FREQ / (modedb[mode].pcd + 2));
	}else{
		clk = AHB_CLK_FREQ;
	}
        vres = modedb[mode].vres;
        vfp = modedb[mode].vfp;
        vsync = modedb[mode].vsync;
        vbp = modedb[mode].vbp;
        vtotal = vres + vfp + vsync + vbp;

        hres = modedb[mode].hres;
        hfp = modedb[mode].hfp;
        hsync = modedb[mode].hsync;
        hbp = modedb[mode].hbp;
        htotal = hres + hfp + hsync + hbp;

        info->var.xres = hres;
        info->var.yres = vres;
        info->var.xres_virtual = hres;
        info->var.yres_virtual = vres;
        info->var.bits_per_pixel = bpp;
        if (bpp == 8) {
                info->var.red.length = 8;
                info->var.green.length = 8;
                info->var.blue.length = 8;
        } else if ( bpp == 16) {
                info->var.red.offset = 10;
                info->var.red.length = 5;
                info->var.green.offset = 5;
                info->var.green.length = 5;
                info->var.blue.offset = 0;
                info->var.blue.length = 5;
        } else if ( bpp == 24 || bpp == 32 ) {
                info->var.red.offset = 16;
                info->var.red.length = 8;
                info->var.green.offset = 8;
                info->var.green.length = 8;
                info->var.blue.offset = 0;
                info->var.blue.length = 8;
        }

        info->var.activate = FB_ACTIVATE_NOW;
	info->var.pixclock = 1000000000 / (clk / 1000);
        info->var.height = modedb[mode].vres;
        info->var.width = modedb[mode].hres;
        info->var.left_margin = hbp;
        info->var.right_margin = hfp;
        info->var.upper_margin = vbp ;
        info->var.lower_margin = vfp;
        info->var.hsync_len = hsync;
        info->var.vsync_len = vsync;
#if 0
        if (modedb[mode].pol_hsync == POL_HIGH)
                info->var.sync |= FB_SYNC_HOR_HIGH_ACT;
        if (modedb[mode].pol_vsync == POL_HIGH)
                info->var.sync |= FB_SYNC_VERT_HIGH_ACT;
#endif
        info->var.vmode = FB_VMODE_NONINTERLACED;

        info->fix.smem_start = fb_phys;
        info->fix.smem_len = fb_size;
        info->fix.type = FB_TYPE_PACKED_PIXELS;
        if (bpp == 8)
                info->fix.visual = FB_VISUAL_PSEUDOCOLOR;
        else if (bpp == 16 || bpp == 24 || bpp == 32)
                info->fix.visual = FB_VISUAL_TRUECOLOR;

        info->fix.line_length = hres * (bpp / 8);
        info->screen_base = fb_log;

	/* Register configuration */
	if(enable)
		ns9750fb_enable(info);
	else
		ns9750fb_disable(info);

        if (modedb[mode].configure)
                (modedb[mode].configure)(info);

        ns9750fb_blank(0, info);

        return 0;
}

static int
ns9750fb_options(void)
{
	int i,j;

#ifndef MODULE
	char *option = NULL;
	char *opt_str;

	if(fb_get_options(DRIVER_NAME, &option))
		return -ENODEV;

	while((opt_str = strsep(&option, ",")) != NULL){
	if(!(*opt_str)){
	    continue;
	  }else if(strcmp(opt_str, "enable") == 0){
	    enable = 1;
	  }else if(strcmp(opt_str, "disable") == 0){
	    enable = 0;
	  }else if(strcmp(opt_str, "8bpp") == 0){
	    bpp = 8;
	  }else if(strcmp(opt_str, "16bpp") == 0){
	    bpp = 16;
	  }else if(strcmp(opt_str, "24bpp") == 0){
	    bpp = 24;
	  }else if(strcmp(opt_str, "32bpp") == 0){
	    bpp = 32;
	  }else{
	    for(i=0; i<ARRAY_SIZE(modedb); i++){
	      for(j=0; modedb[i].name[j]; j++){
		if(strcmp(opt_str, modedb[i].name[j]) == 0){
		  modestr = opt_str;;
		  continue;
		}
	      }
	    }
	    /* Unknown option */
	  }
	}
#endif

	DEBUG_FUNC();

	if(!modestr){
		/* Setup Default Mode */
		modestr = CONFIG_FB_NS9750_DEFAULT_MODE;
	}

	for(i=0; i<ARRAY_SIZE(modedb); i++){
		for(j=0; modedb[i].name[j]; j++){
			if(strcmp(modestr, modedb[i].name[j]) == 0){
				mode = i;
			}
		}
	}
	
	if(mode == -1){
		PRINT_ERR("Not support mode: %s\n", modestr);
		return -1;
	}

	if(bpp == -1){
		bpp = CONFIG_FB_NS9750_DEFAULT_BPP;
	}
	
	switch(bpp){
	case 8:
		break;
	case 16:
		break;
#if defined(CONFIG_FB_NS9750_24BIT)
	case 24:
		PRINT_INFO("Using 32bpp instead of 24bpp.\n");
		bpp = 32;
		break;
	case 32:
		break;
#endif
	default:
		PRINT_ERR("Not support bpp: %d\n", bpp);
		return -1;
	}
  
	DEBUG_INFO("modestr: %s\n", modestr);
	DEBUG_INFO("mode: %d\n", mode);
	DEBUG_INFO("bpp: %d\n", bpp);
	DEBUG_INFO("enable: %d\n", enable);
	
	return 0;
}

#if defined(CONFIG_PROC_FS)
#include <linux/proc_fs.h>

static int 
ns9750fb_proc_info_read(char *page, char **start, off_t off,
			int count, int *eof, void *data)
{
	int ret;
	DEBUG_FUNC();
	ret = sprintf(page,
		      "state: %s\n""mode: %s\n""bpp: %d\n",
		      enable ? "enable" : "disable", modestr, bpp);
	return ret;
}
static int 
ns9750fb_proc_state_read(char *page, char **start, off_t off,
			 int count, int *eof, void *data)
{
	int ret;
	ret = sprintf(page, "%d\n", enable);
	return ret;
}

static int 
ns9750fb_proc_state_write(struct file *file, const char __user *buffer,
			  unsigned long count, void *data)
{
	unsigned long state = simple_strtoul(buffer, NULL, 0);
	struct device *dev = (struct device *)data;
	struct fb_info *info = dev_get_drvdata(dev);
	DEBUG_FUNC();
	switch(state){
	case 0:
		if(enable){
			ns9750fb_disable(info);
			ns9750fb_gpio_release((bpp>16)? 1:0);
			enable = 0;
			PRINT_DEBUG("Disable.\n");
		}
	  break;
	case 1:
		if(!enable){
			ns9750fb_gpio_request((bpp>16) ? 1:0);
			ns9750fb_enable(info);
			enable = 1;
			PRINT_DEBUG("Enable.\n");
		}
		break;
	default:
		return -EINVAL;
	}
	return count;
}

struct proc_dir_entry *proc_driver = NULL;
struct proc_dir_entry *proc_driver_info = NULL;
struct proc_dir_entry *proc_driver_state = NULL;
static void ns9750fb_create_proc_entry(struct device *dev)
{
	DEBUG_FUNC();
	proc_driver = proc_mkdir(DRIVER_NAME, proc_root_driver);
	proc_driver_info = create_proc_entry("info", S_IRUGO, proc_driver);
	if(proc_driver_info){
		proc_driver_info->read_proc = &ns9750fb_proc_info_read;
	}
	proc_driver_state = create_proc_entry("state", S_IRUGO | S_IWUSR, proc_driver);
	if(proc_driver_state){
		proc_driver_state->data = (void *)dev;
		proc_driver_state->read_proc = &ns9750fb_proc_state_read;
		proc_driver_state->write_proc = &ns9750fb_proc_state_write;
	}
}

static void ns9750fb_remove_proc_entry(void)
{
	DEBUG_FUNC();
	remove_proc_entry("info", proc_driver);
	remove_proc_entry("state", proc_driver);
	remove_proc_entry(DRIVER_NAME, proc_root_driver);
}
#else /* defined(CONFIG_PROC_FS) */
#define ns9750fb_create_proc_entry()
#define ns9750fb_remove_proc_entry()
#endif /* defined(CONFIG_PROC_FS) */

static int
ns9750fb_drv_probe(struct device *dev)
{
	struct platform_device *pdev = to_platform_device(dev);
	struct resource *res;
	struct fb_info *info = NULL;
	int ret;
	
	DEBUG_FUNC();
	
	ret = ns9750fb_options();
	if(ret)
		return -EINVAL;

	res = platform_get_resource(pdev, IORESOURCE_MEM, 0);
	if(!res)
		return -ENODEV;
	
	if(!request_mem_region(res->start, res->end - res->start + 1, 
			       DRIVER_NAME))
		return -EBUSY;
  
	if (bpp == 8)
		info = framebuffer_alloc(sizeof(u32) * 256, &pdev->dev);
	else if (bpp == 16 || bpp == 24 || bpp == 32)
		info = framebuffer_alloc(sizeof(u32) * 16, &pdev->dev);
	else {
		ret = -EINVAL;
		goto memregion;
	}
	
        info->flags = FBINFO_DEFAULT;
        strncpy(info->fix.id, DRIVER_NAME, sizeof(info->fix.id));
        info->fix.mmio_start = res->start;
        info->fix.mmio_len = res->end - res->start + 1;
        info->fbops = &ns9750fb_ops;
        info->pseudo_palette = info->par;
        info->state = FBINFO_STATE_RUNNING;

        if (fb_alloc_cmap(&info->cmap, 256, 0) < 0) {
                ret = -ENOMEM;
                goto framebuffer;
        }

	if(enable)
		ns9750fb_gpio_request((bpp>16) ? 1:0);

	if ((ret = ns9750fb_configure(info)) < 0)
		goto colormap;

	if(enable){
		PRINT_DEBUG("Enable.\n");
	}else{
		PRINT_DEBUG("Disable.\n");
	}

        if (register_framebuffer(info) < 0) {
                PRINT_ERR("Unable to register NS9750 frame buffer\n");
                ret = -EINVAL;
                goto dma;
        }

        dev_set_drvdata(dev, info);

	ns9750fb_create_proc_entry(dev);

        printk(KERN_INFO "fb%d: NS9750 frame buffer at %dx%dx%d\n", info->node,
               info->var.xres, info->var.yres, info->var.bits_per_pixel);

        return 0;

dma:
        dma_free_writecombine(NULL, info->fix.smem_len,
                              info->screen_base, info->fix.smem_start);
colormap:
	ns9750fb_gpio_release((bpp>16)? 1:0);
        fb_dealloc_cmap(&info->cmap);
framebuffer:
        framebuffer_release(info);
memregion:
        release_mem_region(res->start, res->end - res->start + 1);

        return ret;
}

static int __devexit
ns9750fb_drv_remove(struct device *dev)
{
	struct platform_device *pdev = to_platform_device(dev);
	struct fb_info *info = dev_get_drvdata(dev);
	struct resource *res;

	DEBUG_FUNC();

	ns9750fb_disable(info);
	if(enable)
		ns9750fb_gpio_release((bpp>16)? 1:0);

	dev_set_drvdata(dev, NULL);

	unregister_framebuffer(info);

	dma_free_writecombine(NULL, info->fix.smem_len,
			      info->screen_base, info->fix.smem_start);
	fb_dealloc_cmap(&info->cmap);
	framebuffer_release(info);

	res = platform_get_resource(pdev, IORESOURCE_MEM, 0);
	release_mem_region(res->start, res->end - res->start + 1);

	ns9750fb_remove_proc_entry();

	return 0;
}

static struct device_driver ns9750fb_driver = {
	.name		= DRIVER_NAME,
	.bus		= &platform_bus_type,
	.probe		= ns9750fb_drv_probe,
	.remove		= __devexit_p(ns9750fb_drv_remove),
};

static int __init
ns9750fb_init(void)
{
	DEBUG_FUNC();
	PRINT_INFO(DESCRIPTION " " REVISION ", (C) 2006-2007 " AUTHOR "\n");
	return driver_register(&ns9750fb_driver);
}

static void __exit
ns9750fb_cleanup(void)
{
	DEBUG_FUNC();
	return driver_unregister(&ns9750fb_driver);
}

module_init(ns9750fb_init);
module_exit(ns9750fb_cleanup);

module_param(modestr, charp, 0);
module_param(bpp, int, 0);
module_param(enable, bool, 0);

MODULE_AUTHOR(AUTHOR);
MODULE_DESCRIPTION(DESCRIPTION);
MODULE_LICENSE("GPL");
