/***********************************************************************
 *
 * Copyright (C) 2004 by FS Forth-Systeme GmbH.
 * All rights reserved.
 *
 * $Id: ns9750_bbus_bridge.h,v 1.2 2005/09/06 07:35:00 uzeisberger Exp $
 * @Author: Markus Pietrek
 * @Descr: Definitions for BBus Bridge usage (DMA and interrupts)
 * @References: [1] NS9750 Hardware Reference Manual/December 2003 Chap. 8
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 ***********************************************************************/
/***********************************************************************
 *  @History:
 *     2004/08/17 : remmoved addr_bbus_
 ***********************************************************************/

#ifndef FS_NS9750_BBUS_BRIDGE_H
#define FS_NS9750_BBUS_BRIDGE_H

#define NS_BBUS_BRIDGE_MODULE_BASE	(0xFA400000) /* virtual address */
#define NS_BBUS_BRIDGE_MODULE_BASE_PA	(0xA0400000) /* physical address */
#define NS_BBUS_BRIDGE_MODULE_SIZE	(0x00100000)

/* offset for DMA channel 1 */

#define NS_BBUS_BRIDGE_DMA_OFFSET	(0x0020)

/* the register addresses */

#define NS_BBUS_BRIDGE_BDP_BASE		(NS_BBUS_BRIDGE_MODULE_BASE + 0x0000)
#define NS_BBUS_BRIDGE_CTRL_BASE	(NS_BBUS_BRIDGE_MODULE_BASE + 0x0004)
#define NS_BBUS_BRIDGE_STAT_BASE	(NS_BBUS_BRIDGE_MODULE_BASE + 0x0008)
#define NS_BBUS_BRIDGE_ISTAT		(NS_BBUS_BRIDGE_MODULE_BASE + 0x1000)
#define NS_BBUS_BRIDGE_IEN		(NS_BBUS_BRIDGE_MODULE_BASE + 0x1004)

/* register bit fields */

#define NS_BBUS_BRIDGE_CTRL_CE		(0x80000000)
#define NS_BBUS_BRIDGE_CTRL_CA 		(0x40000000)
#define NS_BBUS_BRIDGE_CTRL_CG		(0x20000000)
#define NS_BBUS_BRIDGE_CTRL_SB_MA	(0x00180000)
#define NS_BBUS_BRIDGE_CTRL_SB_4	(0x00180000)
#define NS_BBUS_BRIDGE_CTRL_SB_3	(0x00100000)
#define NS_BBUS_BRIDGE_CTRL_SB_2	(0x00080000)
#define NS_BBUS_BRIDGE_CTRL_SB_1	(0x00000000)
#define NS_BBUS_BRIDGE_CTRL_DB_MA	(0x00060000)
#define NS_BBUS_BRIDGE_CTRL_DB_8 	(0x00060000)
#define NS_BBUS_BRIDGE_CTRL_DB_4 	(0x00040000)
#define NS_BBUS_BRIDGE_CTRL_DB_2 	(0x00020000)
#define NS_BBUS_BRIDGE_CTRL_DB_1 	(0x00000000)
#define NS_BBUS_BRIDGE_CTRL_SINC_N	(0x00010000)
#define NS_BBUS_BRIDGE_CTRL_DINC_N	(0x00008000)
#define NS_BBUS_BRIDGE_CTRL_STATE_MA 	(0x0000FC00)
#define NS_BBUS_BRIDGE_CTRL_INDEX_MA 	(0x000003FF)

#define NS_BBUS_BRIDGE_STAT_NCIP	(0x80000000)
#define NS_BBUS_BRIDGE_STAT_ECIP	(0x40000000)
#define NS_BBUS_BRIDGE_STAT_NRIP	(0x20000000)
#define NS_BBUS_BRIDGE_STAT_CAIP	(0x10000000)
#define NS_BBUS_BRIDGE_STAT_PCIP	(0x08000000)
#define NS_BBUS_BRIDGE_STAT_NCIE	(0x01000000)
#define NS_BBUS_BRIDGE_STAT_ECIE	(0x00800000)
#define NS_BBUS_BRIDGE_STAT_NRIE	(0x00400000)
#define NS_BBUS_BRIDGE_STAT_CAIE	(0x00200000)
#define NS_BBUS_BRIDGE_STAT_PCIE	(0x00100000)
#define NS_BBUS_BRIDGE_STAT_WRAP	(0x00080000)
#define NS_BBUS_BRIDGE_STAT_IDONE	(0x00040000)
#define NS_BBUS_BRIDGE_STAT_LAST	(0x00020000)
#define NS_BBUS_BRIDGE_STAT_FULL	(0x00010000)
#define NS_BBUS_BRIDGE_STAT_BLEN_MA	(0x0000FFFF)

#define NS_BBUS_BRIDGE_ISTAT_ADMA2	(0x02000000)
#define NS_BBUS_BRIDGE_ISTAT_ADMA1	(0x01000000)
#define NS_BBUS_BRIDGE_ISTAT_UTIL	(0x00001000)
#define NS_BBUS_BRIDGE_ISTAT_1284	(0x00000800)
#define NS_BBUS_BRIDGE_ISTAT_I2C	(0x00000400)
#define NS_BBUS_BRIDGE_ISTAT_S4TX	(0x00000200)
#define NS_BBUS_BRIDGE_ISTAT_S4RX	(0x00000100)
#define NS_BBUS_BRIDGE_ISTAT_S3TX	(0x00000080)
#define NS_BBUS_BRIDGE_ISTAT_S3RX	(0x00000040)
#define NS_BBUS_BRIDGE_ISTAT_S2TX	(0x00000020)
#define NS_BBUS_BRIDGE_ISTAT_S2RX	(0x00000010)
#define NS_BBUS_BRIDGE_ISTAT_S1TX	(0x00000008)
#define NS_BBUS_BRIDGE_ISTAT_S1RX	(0x00000004)
#define NS_BBUS_BRIDGE_ISTAT_USB	(0x00000002)
#define NS_BBUS_BRIDGE_ISTAT_BBDMA	(0x00000001)

#define NS_BBUS_BRIDGE_IEN_GLBL 	(0x80000000)
#define NS_BBUS_BRIDGE_IEN_ADMA2	(0x02000000)
#define NS_BBUS_BRIDGE_IEN_ADMA1	(0x01000000)
#define NS_BBUS_BRIDGE_IEN_UTIL		(0x00001000)
#define NS_BBUS_BRIDGE_IEN_1284		(0x00000800)
#define NS_BBUS_BRIDGE_IEN_I2C		(0x00000400)
#define NS_BBUS_BRIDGE_IEN_S4TX		(0x00000200)
#define NS_BBUS_BRIDGE_IEN_S4RX		(0x00000100)
#define NS_BBUS_BRIDGE_IEN_S3TX		(0x00000080)
#define NS_BBUS_BRIDGE_IEN_S3RX		(0x00000040)
#define NS_BBUS_BRIDGE_IEN_S2TX		(0x00000020)
#define NS_BBUS_BRIDGE_IEN_S2RX		(0x00000010)
#define NS_BBUS_BRIDGE_IEN_S1TX		(0x00000008)
#define NS_BBUS_BRIDGE_IEN_S1RX		(0x00000004)
#define NS_BBUS_BRIDGE_IEN_USB		(0x00000002)
#define NS_BBUS_BRIDGE_IEN_BBDMA	(0x00000001)

#endif /* FS_NS9750_BBUS_BRIDGE_H */

