/***********************************************************************
 *
 * Copyright (C) 2004 by FS Forth-Systeme GmbH.
 * All rights reserved.
 *
 * $Id: ns9750_mem.h,v 1.2 2005/09/06 07:35:00 uzeisberger Exp $
 * @Author: Markus Pietrek
 * @Descr: Definitions for Memory Control Module
 * @References: [1] NS9750 Hardware Reference Manual/December 2003 Chap. 5
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 *
 ***********************************************************************/
/***********************************************************************
 *  @History:
 *     2004/08/17 : removed addr_mem
 ***********************************************************************/

#ifndef FS_NS9750_MEM_H
#define FS_NS9750_MEM_H

#define NS_MEM_MODULE_BASE		(0xFA700000) /* virtual address */
#define NS_MEM_MODULE_BASE_PA		(0xA0700000) /* physical address */
#define NS_MEM_MODULE_SIZE		(0x00100000)

/* the register addresses */

#define NS_MEM_CTRL     		(NS_MEM_MODULE_BASE + 0x0000)
#define NS_MEM_STATUS     		(NS_MEM_MODULE_BASE + 0x0004)
#define NS_MEM_CFG     			(NS_MEM_MODULE_BASE + 0x0008)
#define NS_MEM_DYN_CTRL 		(NS_MEM_MODULE_BASE + 0x0020)
#define NS_MEM_DYN_REFRESH 		(NS_MEM_MODULE_BASE + 0x0024)
#define NS_MEM_DYN_READ_CFG		(NS_MEM_MODULE_BASE + 0x0028)
#define NS_MEM_DYN_TRP     		(NS_MEM_MODULE_BASE + 0x0030)
#define NS_MEM_DYN_TRAS    		(NS_MEM_MODULE_BASE + 0x0034)
#define NS_MEM_DYN_TSREX   		(NS_MEM_MODULE_BASE + 0x0038)
#define NS_MEM_DYN_TAPR   		(NS_MEM_MODULE_BASE + 0x003C)
#define NS_MEM_DYN_TDAL    		(NS_MEM_MODULE_BASE + 0x0040)
#define NS_MEM_DYN_TWR     		(NS_MEM_MODULE_BASE + 0x0044)
#define NS_MEM_DYN_TRC     		(NS_MEM_MODULE_BASE + 0x0048)
#define NS_MEM_DYN_TRFC    		(NS_MEM_MODULE_BASE + 0x004C)
#define NS_MEM_DYN_TXSR    		(NS_MEM_MODULE_BASE + 0x0050)
#define NS_MEM_DYN_TRRD    		(NS_MEM_MODULE_BASE + 0x0054)
#define NS_MEM_DYN_TMRD    		(NS_MEM_MODULE_BASE + 0x0058)
#define NS_MEM_STAT_EXT_WAIT		(NS_MEM_MODULE_BASE + 0x0080)
#define NS_MEM_DYN_CFG_BASE		(NS_MEM_MODULE_BASE + 0x0100)
#define NS_MEM_DYN_RAS_CAS_BASE		(NS_MEM_MODULE_BASE + 0x0104)
#define NS_MEM_STAT_CFG_BASE		(NS_MEM_MODULE_BASE + 0x0200)
#define NS_MEM_STAT_WAIT_WEN_BASE	(NS_MEM_MODULE_BASE + 0x0204)
#define NS_MEM_STAT_WAIT_OEN_BASE	(NS_MEM_MODULE_BASE + 0x0208)
#define NS_MEM_STAT_WAIT_RD_BASE	(NS_MEM_MODULE_BASE + 0x020C)
#define NS_MEM_STAT_WAIT_PAGE_BASE	(NS_MEM_MODULE_BASE + 0x0210)
#define NS_MEM_STAT_WAIT_WR_BASE	(NS_MEM_MODULE_BASE + 0x0214)
#define NS_MEM_STAT_WAIT_TURN_BASE	(NS_MEM_MODULE_BASE + 0x0218)

/* the vectored register addresses */

#define NS_MEM_DYN_CFG(c)		(NS_MEM_DYN_CFG_BASE + (c)*0x20)
#define NS_MEM_DYN_RAS_CAS(c)		(NS_MEM_DYN_RAS_CAS_BASE + (c)*0x20)
#define NS_MEM_STAT_CFG(c)		(NS_MEM_STAT_CFG_BASE + (c)*0x20)
#define NS_MEM_STAT_WAIT_WEN(c)		(NS_MEM_STAT_WAIT_WEN_BASE+(c)*0x20)
#define NS_MEM_STAT_WAIT_OEN(c)		(NS_MEM_STAT_WAIT_OEN_BASE+(c)*0x20)
#define NS_MEM_STAT_RD(c)		(NS_MEM_STAT_WAIT_RD_BASE+(c)*0x20)
#define NS_MEM_STAT_PAGE(c)		(NS_MEM_STAT_WAIT_PAGE_BASE+(c)*0x20)
#define NS_MEM_STAT_WR(c)		(NS_MEM_STAT_WAIT_WR_BASE+(c)*0x20)
#define NS_MEM_STAT_TURN(c)		(NS_MEM_STAT_WAIT_TURN_BASE+(c)*0x20)

/* register bit fields */

#define NS_MEM_CTRL_L			(0x00000004)
#define NS_MEM_CTRL_M			(0x00000002)
#define NS_MEM_CTRL_E			(0x00000001)

#define NS_MEM_STAT_SA			(0x00000004)
#define NS_MEM_STAT_S			(0x00000002)
#define NS_MEM_STAT_B			(0x00000001)

#define NS_MEM_CFG_CLK			(0x00000010)
#define NS_MEM_CFG_N			(0x00000001)

#define NS_MEM_DYN_CTRL_NRP		(0x00004000)
#define NS_MEM_DYN_CTRL_DP		(0x00002000)
#define NS_MEM_DYN_CTRL_I_MA		(0x00000180)
#define NS_MEM_DYN_CTRL_I_NORMAL	(0x00000000)
#define NS_MEM_DYN_CTRL_I_MODE		(0x00000080)
#define NS_MEM_DYN_CTRL_I_PALL		(0x00000100)
#define NS_MEM_DYN_CTRL_I_NOP		(0x00000180)
#define NS_MEM_DYN_CTRL_BIT5		(0x00000020) /* must be set to 0 */
#define NS_MEM_DYN_CTRL_SR		(0x00000004)
#define NS_MEM_DYN_CTRL_BIT1		(0x00000002) /* must be set to 1 */
#define NS_MEM_DYN_CTRL_CE		(0x00000001)


#define NS_MEM_DYN_REFRESH_MA		(0x000007FF)

#define NS_MEM_DYN_READ_CFG_MA		(0x00000003)
#define NS_MEM_DYN_READ_CFG_DELAY0 	(0x00000001)
#define NS_MEM_DYN_READ_CFG_DELAY1  	(0x00000002)
#define NS_MEM_DYN_READ_CFG_DELAY2	(0x00000003)

#define NS_MEM_DYN_TRP_MA		(0x0000000F)

#define NS_MEM_DYN_TRAS_MA		(0x0000000F)

#define NS_MEM_DYN_TSREX_MA		(0x0000000F)

#define NS_MEM_DYN_TAPR_MA		(0x0000000F)

#define NS_MEM_DYN_TDAL_MA		(0x0000000F)

#define NS_MEM_DYN_TWR_MA		(0x0000000F)

#define NS_MEM_DYN_TRC_MA		(0x0000001F)

#define NS_MEM_DYN_TRFC_MA		(0x0000001F)

#define NS_MEM_DYN_TXSR_MA		(0x0000001F)

#define NS_MEM_DYN_TRRD_MA		(0x0000000F)

#define NS_MEM_DYN_TMRD_MA		(0x0000000F)

#define NS_MEM_STAT_EXTW_WAIT_MA	(0x0000003F)

#define NS_MEM_DYN_CFG_P		(0x00100000)
#define NS_MEM_DYN_CFG_BDMC		(0x00080000)
#define NS_MEM_DYN_CFG_AM		(0x00004000)
#define NS_MEM_DYN_CFG_AM_MA		(0x00001F80)
#define NS_MEM_DYN_CFG_MD		(0x00000018)

#define NS_MEM_DYN_RAS_CAS_CAS_MA	(0x00000300)
#define NS_MEM_DYN_RAS_CAS_CAS_1 	(0x00000100)
#define NS_MEM_DYN_RAS_CAS_CAS_2 	(0x00000200)
#define NS_MEM_DYN_RAS_CAS_CAS_3 	(0x00000300)
#define NS_MEM_DYN_RAS_CAS_RAS_MA	(0x00000003)
#define NS_MEM_DYN_RAS_CAS_RAS_1	(0x00000001)
#define NS_MEM_DYN_RAS_CAS_RAS_2 	(0x00000002)
#define NS_MEM_DYN_RAS_CAS_RAS_3 	(0x00000003)

#define NS_MEM_STAT_CFG_PSMC		(0x00100000)
#define NS_MEM_STAT_CFG_BSMC		(0x00080000)
#define NS_MEM_STAT_CFG_EW		(0x00000100)
#define NS_MEM_STAT_CFG_PB		(0x00000080)
#define NS_MEM_STAT_CFG_PC		(0x00000040)
#define NS_MEM_STAT_CFG_PM		(0x00000008)
#define NS_MEM_STAT_CFG_MW_MA		(0x00000003)
#define NS_MEM_STAT_CFG_MW_8		(0x00000000)
#define NS_MEM_STAT_CFG_MW_16      	(0x00000001)
#define NS_MEM_STAT_CFG_MW_32		(0x00000002)

#define NS_MEM_STAT_WAIT_WEN_MA		(0x0000000F)

#define NS_MEM_STAT_WAIT_OEN_MA		(0x0000000F)

#define NS_MEM_STAT_WAIT_RD_MA		(0x0000001F)

#define NS_MEM_STAT_WAIT_PAGE_MA	(0x0000001F)

#define NS_MEM_STAT_WAIT_WR_MA		(0x0000001F)

#define NS_MEM_STAT_WAIT_TURN_MA	(0x0000000F)


#endif /* FS_NS9750_MEM_H */
