/***********************************************************************
 *
 * Copyright (C) 2004 by FS Forth-Systeme GmbH.
 * All rights reserved.
 *
 * $Id: ns9750_ser.h,v 1.3 2005/09/06 07:35:00 uzeisberger Exp $
 * @Author: Markus Pietrek
 * @References: [1] NS9750 Hardware Reference, December 2003
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 ***********************************************************************/
/***********************************************************************
 *  @History:
 *     2005/07/18 : Added masks to handle different SPI modes
 *     2005/05/06 : fixed bug #395
 *     2004/08/17 : removed addr_ser
 ***********************************************************************/

#ifndef FS_NS9750_SER_H
#define FS_NS9750_SER_H

# define NS_SER_MODULE_BASE	 	(0xF9200000) /* virtual address */
# define NS_SER_MODULE_BASE_PA		(0x90200000) /* physical address */
# define NS_SER_MODULE_SIZE	 	(0x00200000)

# define NS_SER_CHAN_OFFSET		(0x00100000)

/*
 * ttyAM0 - UARTA
 * ttyAM1 - UARTB
 * ttyAM2 - UARTC
 * ttyAM3 - UARTD
 */

#define addr_ser_channel(reg,chan) \
( \
  (reg) + \
  (((chan) < 2) ? \
    (((chan) & 1) ? 0 : 0x40) : \
    (((chan) & 1) ? (0x40 + NS_SER_CHAN_OFFSET): (NS_SER_CHAN_OFFSET))) \
)

#define irqno_ser_channel(chan) \
( \
  (IRQ_BBUS_S1RXE) + \
  ((chan < 2) ? \
  ((chan & 1) ? 0 : 2) : (chan * 2)) \
)

/* use get_ser_reg_c and get_ser_fifo_channel for character acceses to BBUS */

#define get_ser_reg(c) \
 (*(volatile unsigned int *)((unsigned int) (c)))

#define get_ser_reg_channel(reg,chan) \
 get_ser_reg(addr_ser_channel(reg, chan))

#define get_ser_reg_c(c) \
 (*(volatile char *)((unsigned int) (c)))

/* the register addresses */

#define NS_SER_CTRL_A   (NS_SER_MODULE_BASE + 0x00)
#define NS_SER_CTRL_B   (NS_SER_MODULE_BASE + 0x04)
#define NS_SER_STAT_A   (NS_SER_MODULE_BASE + 0x08)
#define NS_SER_BITRATE   (NS_SER_MODULE_BASE + 0x0C)
#define NS_SER_FIFO   (NS_SER_MODULE_BASE + 0x10)
#define NS_SER_RX_BUF_TIMER  (NS_SER_MODULE_BASE + 0x14)
#define NS_SER_RX_CHAR_TIMER  (NS_SER_MODULE_BASE + 0x18)
#define NS_SER_RX_MATCH   (NS_SER_MODULE_BASE + 0x1C)
#define NS_SER_RX_MATCH_MASK  (NS_SER_MODULE_BASE + 0x20)
#define NS_SER_FLOW_CTRL  (NS_SER_MODULE_BASE + 0x34)
#define NS_SER_FLOW_CTRL_FORCE  (NS_SER_MODULE_BASE + 0x38)

/* register bit fields */

/* control A register */

#define NS_SER_CTRL_A_CE  (0x80000000)
#define NS_SER_CTRL_A_BRK  (0x40000000)
#define NS_SER_CTRL_A_STICKP  (0x20000000)
#define NS_SER_CTRL_A_EPS  (0x10000000)
#define NS_SER_CTRL_A_PE  (0x08000000)
#define NS_SER_CTRL_A_STOP  (0x04000000)
#define NS_SER_CTRL_A_WLS_MA  (0x03000000)
#define NS_SER_CTRL_A_WLS_5  (0x00000000)
#define NS_SER_CTRL_A_WLS_6  (0x01000000)
#define NS_SER_CTRL_A_WLS_7  (0x02000000)
#define NS_SER_CTRL_A_WLS_8  (0x03000000)
#define NS_SER_CTRL_A_CTSTX  (0x00800000)
#define NS_SER_CTRL_A_RTSRX  (0x00400000)
#define NS_SER_CTRL_A_RL  (0x00200000)
#define NS_SER_CTRL_A_LL  (0x00100000)
#define NS_SER_CTRL_A_RES  (0x000CF000)
#define NS_SER_CTRL_A_DTR  (0x00020000)
#define NS_SER_CTRL_A_RTS  (0x00010000)
#define NS_SER_CTRL_A_RIE_MA  (0x00000E00)
#define NS_SER_CTRL_A_RIE_RDY  (0x00000800)
#define NS_SER_CTRL_A_RIE_HALF  (0x00000400)
#define NS_SER_CTRL_A_RIE_CLOSED (0x00000200)
#define NS_SER_CTRL_A_ERXDMA     (0x00000100)
#define NS_SER_CTRL_A_RIC_MA  (0x000000E0)
#define NS_SER_CTRL_A_TIC_MA  (0x0000001E)
#define NS_SER_CTRL_A_TIC_CTR  (0x00000010)
#define NS_SER_CTRL_A_TIC_EMPTY  (0x00000008)
#define NS_SER_CTRL_A_TIC_HALF  (0x00000004)
#define NS_SER_CTRL_A_TIC_CLOSED (0x00000002)
#define NS_SER_CTRL_A_ETXDMA   (0x00000001)

/* control B register */

#define NS_SER_CTRL_B_RDM1  (0x80000000)
#define NS_SER_CTRL_B_RDM2  (0x40000000)
#define NS_SER_CTRL_B_RDM3  (0x20000000)
#define NS_SER_CTRL_B_RDM4  (0x10000000)
#define NS_SER_CTRL_B_RBGT  (0x08000000)
#define NS_SER_CTRL_B_RCGT  (0x04000000)
#define NS_SER_CTRL_B_MODE_MA  (0x00300000)
#define NS_SER_CTRL_B_MODE_UART  (0x00000000)
#define NS_SER_CTRL_B_MODE_HDLC  (0x00100000)
#define NS_SER_CTRL_B_MODE_SPI_M (0x00200000)
#define NS_SER_CTRL_B_MODE_SPI_S (0x00300000)
#define NS_SER_CTRL_B_BITORDR  (0x00080000)
#define NS_SER_CTRL_B_RES  (0x0007703F)
#define NS_SER_CTRL_B_RTSTX  (0x00008000)
#define NS_SER_CTRL_B_ENDEC_MA  (0x00000FC0)

/* status A register */

#define NS_SER_STAT_A_MATCH1  (0x80000000)
#define NS_SER_STAT_A_MATCH2  (0x40000000)
#define NS_SER_STAT_A_MATCH3  (0x20000000)
#define NS_SER_STAT_A_MATCH4  (0x10000000)
#define NS_SER_STAT_A_BGAP  (0x08000000)
#define NS_SER_STAT_A_CGAP  (0x04000000)
#define NS_SER_STAT_A_RXFDB_MA  (0x00300000)
#define NS_SER_STAT_A_RXFDB_FULL (0x00000000)
#define NS_SER_STAT_A_RXFDB_1  (0x00100000)
#define NS_SER_STAT_A_RXFDB_2  (0x00200000)
#define NS_SER_STAT_A_RXFDB_3  (0x00300000)
#define NS_SER_STAT_A_DCD  (0x00080000)
#define NS_SER_STAT_A_RI  (0x00040000)
#define NS_SER_STAT_A_DSR  (0x00020000)
#define NS_SER_STAT_A_CTS  (0x00010000)
#define NS_SER_STAT_A_RBRK  (0x00008000)
#define NS_SER_STAT_A_RFE  (0x00004000)
#define NS_SER_STAT_A_RPE  (0x00002000)
#define NS_SER_STAT_A_ROVER  (0x00001000)
#define NS_SER_STAT_A_RRDY  (0x00000800)
#define NS_SER_STAT_A_RHALF  (0x00000400)
#define NS_SER_STAT_A_RBC  (0x00000200)
#define NS_SER_STAT_A_RFULL  (0x00000100)
#define NS_SER_STAT_A_DCDI  (0x00000080)
#define NS_SER_STAT_A_RII  (0x00000040)
#define NS_SER_STAT_A_DSRI  (0x00000020)
#define NS_SER_STAT_A_CTSI  (0x00000010)
#define NS_SER_STAT_A_TRDY  (0x00000008)
#define NS_SER_STAT_A_THALF  (0x00000004)
#define NS_SER_STAT_A_TBC  (0x00000002)
#define NS_SER_STAT_A_TEMPTY  (0x00000001)

#define NS_SER_STAT_A_RX_COND_ERR ( NS_SER_STAT_A_RFE | \
             NS_SER_STAT_A_ROVER | \
            NS_SER_STAT_A_RPE )
#define NS_SER_STAT_A_RX_COND_ALL ( NS_SER_STAT_A_RX_COND_ERR | \
            NS_SER_STAT_A_RBRK | \
            NS_SER_STAT_A_RRDY | \
         NS_SER_STAT_A_RHALF | \
         NS_SER_STAT_A_RBC | \
         NS_SER_STAT_A_DCDI | \
         NS_SER_STAT_A_RII | \
         NS_SER_STAT_A_DSRI | \
         NS_SER_STAT_A_CTSI )
#define NS_SER_STAT_A_TX_COND_ALL ( NS_SER_STAT_A_TRDY | \
            NS_SER_STAT_A_THALF | \
            NS_SER_STAT_A_TBC | \
         NS_SER_STAT_A_TEMPTY )
/* bit rate register */

#define NS_SER_BITRATE_EBIT  (0x80000000)
#define NS_SER_BITRATE_TMODE   (0x40000000)
#define NS_SER_BITRATE_RXSRC   (0x20000000)
#define NS_SER_BITRATE_TXSRC   (0x10000000)
#define NS_SER_BITRATE_RXEXT   (0x08000000)
#define NS_SER_BITRATE_TXEXT   (0x04000000)
#define NS_SER_BITRATE_CLKMUX_MA (0x03000000)
#define NS_SER_BITRATE_CLKMUX_XTAL (0x00000000)
#define NS_SER_BITRATE_CLKMUX_BCLK (0x01000000)
#define NS_SER_BITRATE_CLKMUX_OUT1 (0x02000000)
#define NS_SER_BITRATE_CLKMUX_OUT2 (0x03000000)
#define NS_SER_BITRATE_MOD_MA   (0x00E00000)
#define NS_SER_BITRATE_TXCINV   (0x00800000)
#define NS_SER_BITRATE_RXCINV   (0x00400000)
#define NS_SER_BITRATE_SPCPOL_MA   (0x00200000)
#define NS_SER_BITRATE_SPCPOL_LOW   (0x00200000)
#define NS_SER_BITRATE_TCDR_MA   (0x00180000)
#define NS_SER_BITRATE_TCDR_1    (0x00000000)
#define NS_SER_BITRATE_TCDR_8    (0x00080000)
#define NS_SER_BITRATE_TCDR_16   (0x00100000)
#define NS_SER_BITRATE_TCDR_32   (0x00180000)
#define NS_SER_BITRATE_RCDR_MA   (0x00070000)
#define NS_SER_BITRATE_RCDR_1    (0x00000000)
#define NS_SER_BITRATE_RCDR_8    (0x00020000)
#define NS_SER_BITRATE_RCDR_16   (0x00040000)
#define NS_SER_BITRATE_RCDR_32   (0x00060000)
#define NS_SER_BITRATE_TICS  (0x00010000)
#define NS_SER_BITRATE_RICS  (0x00008000)
#define NS_SER_BITRATE_N_MA  (0x00007FFF)

/* receive buffer gap timer */

#define NS_SER_RX_BUF_TIMER_TRUN (0x80000000) /* UART and SPI */
#define NS_SER_RX_BUF_TIMER_BT_MA (0x0000FFFF) /* UART and SPI */
#define NS_SER_RX_BUF_TIMER_MAXLEN_MA  (0x0000FFFF) /* HDLC only */

/* receive character gap timer */

#define NS_SER_RX_CHAR_TIMER_TRUN (0x80000000)
#define NS_SER_RX_CHAR_TIMER_CT_MA (0x000FFFFF)

/* receive match */

#define NS_SER_RX_MATCH_RDMB1_MA (0xFF000000)
#define NS_SER_RX_MATCH_RDMB2_MA (0x00FF0000)
#define NS_SER_RX_MATCH_RDMB3_MA (0x0000FF00)
#define NS_SER_RX_MATCH_RDMB4_MA (0x000000FF)

/* receive match mask */

#define NS_SER_RX_MATCH_MASK_RDMB1_MA  (0xFF000000)
#define NS_SER_RX_MATCH_MASK_RDMB2_MA  (0x00FF0000)
#define NS_SER_RX_MATCH_MASK_RDMB3_MA  (0x0000FF00)
#define NS_SER_RX_MATCH_MASK_RDMB4_MA  (0x000000FF)

#endif /* FS_NS9750_SER_H */
